//------------------------------------------------------------------
#property copyright "mladen"
#property link      "www.forex-tsd.com"
//------------------------------------------------------------------

#property indicator_separate_window
#property indicator_buffers   5
#property indicator_plots     3
#property indicator_label1    "Bollinger bands of MACD"
#property indicator_type1     DRAW_FILLING
#property indicator_color1    C'255,238,210'
#property indicator_label2    "MACD"
#property indicator_type2     DRAW_COLOR_LINE
#property indicator_color2    clrLimeGreen,clrLime,clrMediumOrchid,clrMagenta
#property indicator_width2    3
#property indicator_label3    "MACD signal"
#property indicator_type3     DRAW_LINE
#property indicator_color3    clrDimGray
#property indicator_width3    1

//
//
//
//
//

enum enPrices
{
   pr_close,      // Close
   pr_open,       // Open
   pr_high,       // High
   pr_low,        // Low
   pr_median,     // Median
   pr_typical,    // Typical
   pr_weighted,   // Weighted
   pr_average,    // Average
};

input int      FastEMA   = 12;       // MACD fast EMA period
input int      SlowEMA   = 26;       // MACD slow EMA period
input int      SignalEMA =  9;       // Signal EMA period
input enPrices Price     = pr_close; // Price 
input double   StDv      = 2.0;      // Bands deviation multiplier


//
//
//
//
//

double BBUpperBand[];
double BBLowerBand[];
double Macd[];
double MacdColors[];
double signalBuffer[];

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//

int OnInit()
{
   SetIndexBuffer(0,BBUpperBand ,INDICATOR_DATA);
   SetIndexBuffer(1,BBLowerBand ,INDICATOR_DATA);
   SetIndexBuffer(2,Macd        ,INDICATOR_DATA);
   SetIndexBuffer(3,MacdColors  ,INDICATOR_COLOR_INDEX);
   SetIndexBuffer(4,signalBuffer,INDICATOR_DATA);

   //
   //
   //
   //
   //
     
   IndicatorSetString(INDICATOR_SHORTNAME,"BB macd ("+(string)FastEMA+","+(string)SlowEMA+","+(string)SignalEMA+")");
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//

int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime& time[],
                const double& open[],
                const double& high[],
                const double& low[],
                const double& close[],
                const long& tick_volume[],
                const long& volume[],
                const int& spread[])
{

   //
   //
   //
   //
   //

      for (int i=(int)MathMax(prev_calculated-1,1); i<rates_total; i++)
      {
         double price    = getPrice(Price,open,close,high,low,i,rates_total);
         Macd[i]         = iEma(price,FastEMA,i,rates_total,0)-iEma(price,SlowEMA,i,rates_total,1);
         signalBuffer[i] = iEma(Macd[i],SignalEMA,i,rates_total,2);
         double standardDeviation = iDeviation(Macd, signalBuffer[i], SignalEMA, i);
                  BBUpperBand[i]  = signalBuffer[i] + (StDv * standardDeviation);
                  BBLowerBand[i]  = signalBuffer[i] - (StDv * standardDeviation);

         //
         //
         //
         //
         //

         MacdColors[i] = MacdColors[i-1];
            if (Macd[i]>Macd[i-1]) MacdColors[i] = (Macd[i]<=BBUpperBand[i] && Macd[i]>=BBLowerBand[i]) ? 0 : 1;
            if (Macd[i]<Macd[i-1]) MacdColors[i] = (Macd[i]<=BBUpperBand[i] && Macd[i]>=BBLowerBand[i]) ? 2 : 3;
                     
      }
      return(rates_total);
}


//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

double workEma[][3];
double iEma(double price, double period, int r, int total, int instanceNo=0)
{
   if (ArrayRange(workEma,0)!= total) ArrayResize(workEma,total);
   if (r<2 || (period<=1)) { workEma[r][instanceNo]=price; return(price); }

   //
   //
   //
   //
   //
      
   double alpha = 2.0 / (1.0+period);
          workEma[r][instanceNo] = workEma[r-1][instanceNo]+alpha*(price-workEma[r-1][instanceNo]);
   return(workEma[r][instanceNo]);
} 

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

double iDeviation(double& array[],double dMA, int period,int shift)
{
   double dSum = 0.00;
      for(int i=0; i<period && shift-i>=0; i++) dSum += (array[shift-i]-dMA)*(array[shift-i]-dMA);
   return(MathSqrt(dSum/period));
}

//------------------------------------------------------------------
//                                                                  
//------------------------------------------------------------------
//
//
//
//
//


double getPrice(enPrices price, const double& open[], const double& close[], const double& high[], const double& low[], int i, int bars)
{
   switch (price)
   {
      case pr_close:     return(close[i]);
      case pr_open:      return(open[i]);
      case pr_high:      return(high[i]);
      case pr_low:       return(low[i]);
      case pr_median:    return((high[i]+low[i])/2.0);
      case pr_typical:   return((high[i]+low[i]+close[i])/3.0);
      case pr_weighted:  return((high[i]+low[i]+close[i]+close[i])/4.0);
      case pr_average:   return((high[i]+low[i]+close[i]+open[i])/4.0);
   }
   return(0);
}